@@ -4,12 +4,9 @@
 # Distributed under the terms of the Modified BSD License.
 
 
-import errno
 import io
 import os
 import shutil
-import stat
-import warnings
 import mimetypes
 import nbformat
 
@@ -19,32 +16,25 @@ from .filecheckpoints import FileCheckpoints
 from .fileio import FileManagerMixin
 from .manager import ContentsManager
 
-from ipython_genutils.importstring import import_item
-from traitlets import Any, Unicode, Bool, TraitError, observe, default, validate
-from ipython_genutils.py3compat import getcwd, string_types
 
-from notebook import _tz as tz
+from ipython_genutils.importstring import import_item
+from traitlets import Any, Unicode, Bool, TraitError
+from ipython_genutils.py3compat import getcwd, string_types
+from . import tz
 from notebook.utils import (
-    is_hidden, is_file_hidden,
+    is_hidden,
     to_api_path,
 )
 
-try:
-    from os.path import samefile
-except ImportError:
-    # windows + py2
-    from notebook.utils import samefile_simple as samefile
-
 _script_exporter = None
 
 
 def _post_save_script(model, os_path, contents_manager, **kwargs):
     """convert notebooks to Python script after save with nbconvert
 
-    replaces `jupyter notebook --script`
+    replaces `ipython notebook --script`
     """
     from nbconvert.exporters.script import ScriptExporter
-    warnings.warn("`_post_save_script` is deprecated and will be removed in Notebook 5.0", DeprecationWarning)
 
     if model['type'] != 'notebook':
         return
@@ -55,6 +45,7 @@ def _post_save_script(model, os_path, contents_manager, **kwargs):
     log = contents_manager.log
 
     base, ext = os.path.splitext(os_path)
+    py_fname = base + '.py'
     script, resources = _script_exporter.from_filename(os_path)
     script_fname = base + resources.get('output_extension', '.txt')
     log.info("Saving script /%s", to_api_path(script_fname, contents_manager.root_dir))
@@ -66,35 +57,30 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
     root_dir = Unicode(config=True)
 
-    @default('root_dir')
-    def _default_root_dir(self):
+    def _root_dir_default(self):
         try:
             return self.parent.notebook_dir
         except AttributeError:
             return getcwd()
 
-    save_script = Bool(False, config=True, help='DEPRECATED, use post_save_hook. Will be removed in Notebook 5.0')
-
-    @observe('save_script')
-    def _update_save_script(self):
-        self.log.warning("""
-        `--script` is deprecated and will be removed in notebook 5.0.
-
-        You can trigger nbconvert via pre- or post-save hooks:
+    save_script = Bool(False, config=True, help='DEPRECATED, use post_save_hook')
+    def _save_script_changed(self):
+        self.log.warn("""
+        `--script` is deprecated. You can trigger nbconvert via pre- or post-save hooks:
 
             ContentsManager.pre_save_hook
             FileContentsManager.post_save_hook
 
         A post-save hook has been registered that calls:
 
-            jupyter nbconvert --to script [notebook]
+            ipython nbconvert --to script [notebook]
 
         which behaves similarly to `--script`.
         """)
 
         self.post_save_hook = _post_save_script
 
-    post_save_hook = Any(None, config=True, allow_none=True,
+    post_save_hook = Any(None, config=True,
         help="""Python callable or importstring thereof
 
         to be called on the path of a file just saved.
@@ -111,15 +97,12 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         - contents_manager: this ContentsManager instance
         """
     )
-
-    @validate('post_save_hook')
-    def _validate_post_save_hook(self, proposal):
-        value = proposal['value']
-        if isinstance(value, string_types):
-            value = import_item(value)
-        if not callable(value):
-            raise TraitError("post_save_hook must be callable")
-        return value
+    def _post_save_hook_changed(self, name, old, new):
+        if new and isinstance(new, string_types):
+            self.post_save_hook = import_item(self.post_save_hook)
+        elif new:
+            if not callable(new):
+                raise TraitError("post_save_hook must be callable")
 
     def run_post_save_hook(self, model, os_path):
         """Run the post-save hook if defined, and log errors"""
@@ -127,22 +110,18 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
             try:
                 self.log.debug("Running post-save hook on %s", os_path)
                 self.post_save_hook(os_path=os_path, model=model, contents_manager=self)
-            except Exception as e:
-                self.log.error("Post-save hook failed o-n %s", os_path, exc_info=True)
-                raise web.HTTPError(500, u'Unexpected error while running post hook save: %s' % e)
+            except Exception:
+                self.log.error("Post-save hook failed on %s", os_path, exc_info=True)
 
-    @validate('root_dir')
-    def _validate_root_dir(self, proposal):
+    def _root_dir_changed(self, name, old, new):
         """Do a bit of validation of the root_dir."""
-        value = proposal['value']
-        if not os.path.isabs(value):
+        if not os.path.isabs(new):
             # If we receive a non-absolute path, make it absolute.
-            value = os.path.abspath(value)
-        if not os.path.isdir(value):
-            raise TraitError("%r is not a directory" % value)
-        return value
+            self.root_dir = os.path.abspath(new)
+            return
+        if not os.path.isdir(new):
+            raise TraitError("%r is not a directory" % new)
 
-    @default('checkpoints_class')
     def _checkpoints_class_default(self):
         return FileCheckpoints
 
@@ -267,28 +246,15 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
             model['content'] = contents = []
             os_dir = self._get_os_path(path)
             for name in os.listdir(os_dir):
-                try:
-                    os_path = os.path.join(os_dir, name)
-                except UnicodeDecodeError as e:
-                    self.log.warning(
-                        "failed to decode filename '%s': %s", name, e)
+                os_path = os.path.join(os_dir, name)
+                # skip over broken symlinks in listing
+                if not os.path.exists(os_path):
+                    self.log.warn("%s doesn't exist", os_path)
                     continue
-
-                try:
-                    st = os.stat(os_path)
-                except OSError as e:
-                    # skip over broken symlinks in listing
-                    if e.errno == errno.ENOENT:
-                        self.log.warning("%s doesn't exist", os_path)
-                    else:
-                        self.log.warning("Error stat-ing %s: %s", (os_path, e))
-                    continue
-
-                if not stat.S_ISREG(st.st_mode) and not stat.S_ISDIR(st.st_mode):
+                elif not os.path.isfile(os_path) and not os.path.isdir(os_path):
                     self.log.debug("%s not a regular file", os_path)
                     continue
-
-                if self.should_list(name) and not is_file_hidden(os_path, stat_res=st):
+                if self.should_list(name) and not is_hidden(os_path, self.root_dir):
                     contents.append(self.get(
                         path='%s/%s' % (path, name),
                         content=False)
@@ -486,7 +452,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         old_os_path = self._get_os_path(old_path)
 
         # Should we proceed with the move?
-        if os.path.exists(new_os_path) and not samefile(old_os_path, new_os_path):
+        if os.path.exists(new_os_path):
             raise web.HTTPError(409, u'File already exists: %s' % new_path)
 
         # Move the file
@@ -503,8 +469,6 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
     def get_kernel_path(self, path, model=None):
         """Return the initial API path of  a kernel associated with a given notebook"""
-        if self.dir_exists(path):
-            return path
         if '/' in path:
             parent_dir = path.rsplit('/', 1)[0]
         else:
